@@ -1,7 +1,7 @@
 __author__ = 'Altertech, https://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2020 Altertech'
 __license__ = 'MIT'
-__version__ = '0.0.10'
+__version__ = '0.0.14'
 
 # TODO: 2fa
 
@@ -260,6 +260,26 @@ def set_user_password(password):
         raise AccessDenied
 
 
+def regenerate_user_api_key():
+    """
+    Generate new API key for user
+
+    Returns:
+        new API key
+    Raises:
+        LookupError: user not found
+        webauth.AccessDenied: user is not logged in
+    """
+    user_id = get_user_id()
+    if user_id:
+        k = gen_random_str()
+        _d.db.query('user.api_key.set', _cr=True, id=user_id, api_key=k)
+        _log_user_event('api_key.change')
+        return k
+    else:
+        raise AccessDenied
+
+
 def change_user_email(email, next_action_uri_oldaddr=None,
                       next_action_uri=None):
     """
@@ -376,11 +396,18 @@ def get_user_id():
 
 def get_user_picture():
     """
-    Get picture of current logged in user
+    Get picture of current logged in user (OAuth login only)
     """
     return session.get(f'{_d.x_prefix}user_picture')
 
 
+def get_user_name():
+    """
+    Get name of current logged in user (OAuth login only)
+    """
+    return session.get(f'{_d.x_prefix}user_name')
+
+
 def is_authenticated():
     """
     True if user is logged in
@@ -431,14 +458,17 @@ def _handle_user_auth(user_info, provider):
         if not user_id:
             if allow_registration:
                 user_id = _d.db.qcreate('user.create.empty',
+                                        api_key=gen_random_str(),
                                         d_created=datetime.datetime.now())
             else:
                 raise AccessDenied
+        name = user_info.preferred_username if provider == 'github' \
+                else user_info.name
         _d.db.query('user.provider.create',
                     id=user_id,
                     provider=provider,
                     sub=user_info.sub,
-                    name=user_info.name)
+                    name=name)
         _log_user_event('account.register', user_id=user_id)
         _call_handler('account.register', user_id=user_id, user_info=user_info)
     return user_id
@@ -579,6 +609,7 @@ def register(email, password, confirmed=True, next_action_uri=None):
                                     email=email,
                                     password=sha256(
                                         password.encode()).hexdigest(),
+                                    api_key=gen_random_str(),
                                     d_created=datetime.datetime.now(),
                                     confirmed=confirmed)
         except sqlalchemy.exc.IntegrityError as e:
@@ -599,17 +630,32 @@ def register(email, password, confirmed=True, next_action_uri=None):
 def get_user_email():
     """
     Get email address of current user
+    """
+    try:
+        return _d.db.qlookup('user.get.email',
+                             id=session[f'{_d.x_prefix}user_id'])['email']
+    except LookupError:
+        return None
+
+
+def get_user_api_key():
+    """
+    Get API key of current user
 
     Raises:
         webauth.AccessDenied: if user no longer exists in database
     """
     try:
-        return _d.db.qlookup('user.select.email',
-                             id=session[f'{_d.x_prefix}user_id'])['email']
+        return _d.db.qlookup('user.get.api_key',
+                             id=session[f'{_d.x_prefix}user_id'])['api_key']
     except LookupError:
         raise AccessDenied
 
 
+def get_user_by_api_key(api_key):
+    return _d.db.qlookup('user.select.byapi_key', api_key=api_key)
+
+
 def resend_email_confirm(next_action_uri=None):
     """
     Re-send confirmation for the user email address
@@ -749,8 +795,7 @@ def init(app,
         app.config['SECRET_KEY'] = gen_random_str()
 
     _d.x_prefix, _d.dot_prefix = _format_prefix(base_prefix)
-    _d.db = db.clone()
-    _d.db.rq_func = rq
+    _d.db = db.clone(rq_func=rq)
     _d.kv = KVStorage(db=db, table_name='webauth_kv')
     _d.root_uri = root_uri
     _d.smtp = smtp
@@ -768,7 +813,9 @@ def init(app,
             'webauth_user', meta,
             Column('id', BigInteger(), primary_key=True, autoincrement=True),
             Column('email', VARCHAR(255), nullable=True, unique=True),
-            Column('password', VARCHAR(64), nullable=True),
+            Column('password', CHAR(64), nullable=True),
+            Column('api_key', CHAR(32), nullable=True, unique=True),
+            Index('webauth_user_api_key', 'api_key'),
             Column('d_created', DateTime(timezone=True), nullable=False),
             Column('d_active', DateTime(timezone=True), nullable=True),
             Column('confirmed', Boolean, nullable=False, server_default='0'),
@@ -807,6 +854,7 @@ def init(app,
                 touch(user_id)
                 session[f'{_d.x_prefix}user_id'] = user_id
                 session[f'{_d.x_prefix}user_picture'] = user_info.picture
+                session[f'{_d.x_prefix}user_name'] = user_info.name
                 session[f'{_d.x_prefix}user_confirmed'] = True
                 _call_handler('account.login', user_id=user_id)
                 _log_user_event(f'account.login:{provider}')
